﻿using Hotline.Configurations;
using Hotline.Orders;
using Hotline.Settings;
using Hotline.Settings.TimeLimitDomain;
using Hotline.Share.Dtos.Settings;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Settings;
using Hotline.Share.Tools;
using Mapster;
using Shouldly;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XF.Domain.Repository;

namespace Hotline.Application.Tests.Domain;
public class YiBinExpireTimeTest
{
    private readonly YiBinExpireTimeLimit _calcExpireTime;
    private readonly IRepository<Order> _orderRepository;
    private readonly IRepository<TimeLimitSetting> _timeLimitSettingRepository;

    public YiBinExpireTimeTest(YiBinExpireTimeLimit calcExpireTime, IRepository<Order> orderRepository, IRepository<TimeLimitSetting> timeLimitSettingRepository)
    {
        _calcExpireTime = calcExpireTime;
        _orderRepository = orderRepository;
        _timeLimitSettingRepository = timeLimitSettingRepository;
    }

    [Theory]
    [InlineData("2024-09-04 14:00:00", 2, "10", "2024-09-06 14:00:00", "2024/9/6 10:24:00", "2024/9/5 14:00:00", "2个工作日")]
    [InlineData("2024-09-04 14:01:01", 3, "10", "2024-09-09 14:01:01", "2024/9/6 17:37:01", "2024/9/6 9:31:01", "3个工作日")]
    public async Task CalcEndTime_Test(string begin, int count, string busCode, string expiredTime, string nearlyExpiredTime, string nearlyExpiredTimeOne, string timeText)
    {
        var beginTime = DateTime.Parse(begin);
        var result = await _calcExpireTime.CalcEndTime(beginTime, new TimeConfig(count, ETimeType.WorkDay), busCode);
        result.ShouldNotBeNull();
        result.ExpiredTime.ShouldBe(DateTime.Parse(expiredTime));
        result.NearlyExpiredTime.ShouldBe(DateTime.Parse(nearlyExpiredTime));
        result.NearlyExpiredTimeOne.ShouldBe(DateTime.Parse(nearlyExpiredTimeOne));
        result.Count.ShouldBe(count);
        result.TimeText.ShouldBe(timeText);
    }

    [Theory]
    [InlineData("2024-09-04 14:00:00", "CenterToOrg", "08dccc8f-37b0-40d8-8112-1afb2230c5a3", "2024-09-05 14:00:00")]
    public async Task CalcExpiredTime_Test(string beginTxt, string flowTxt, string orderId, string expected)
    {
        var beginTime = DateTime.Parse(beginTxt);
        if (orderId.Equals("08dccc8f-37b0-40d8-8112-1afb2230c5a3"))
            await InitOrderData(orderId);
        var order = await _orderRepository.Queryable().Where(m => m.Id == orderId).FirstAsync();
        Enum.TryParse(flowTxt, out EFlowDirection flow);
        var time = await _calcExpireTime.CalcExpiredTime(beginTime, flow, order.Adapt<OrderTimeClacInfo>());
        time.ShouldNotBeNull();
        time.ExpiredTime.ShouldBe(DateTime.Parse(expected));
    }

    public async Task InitOrderData(string orderId)
    {
        var order = await _orderRepository.Queryable().Where(m => m.Id == orderId).FirstAsync();
        if (order.Is24HoursComplete) return;
        order.Is24HoursComplete = true;
        await _orderRepository.UpdateAsync(order);
    }

    [Fact]
    public async Task InitExpireTime_Test()
    {
        var entity = new TimeLimitSetting() { BusCode = "YQ", BusName = "疫情", TimeType = ETimeType.WorkDay, TimeValue = 2, Percentage = 80, PercentageOne = 50 };
        if (await _timeLimitSettingRepository.Queryable().Where(m => m.BusCode == entity.BusCode).FirstAsync() == null)
            await _timeLimitSettingRepository.AddAsync(entity);
        entity = new TimeLimitSetting() { BusCode = "24", BusName = "24小时", TimeType = ETimeType.Hour, TimeValue = 24, Percentage = 80, PercentageOne = 50 };
        if (await _timeLimitSettingRepository.Queryable().Where(m => m.BusCode == entity.BusCode).FirstAsync() == null)
            await _timeLimitSettingRepository.AddAsync(entity);
    }
}
